@@ -1,5 +1,14 @@
 void osp_croak(const char* pat, ...);
 
+struct osp_bridge {
+  osp_bridge *next;
+  osp_bridge();
+  virtual ~osp_bridge();
+  virtual void release();		// if perl REFCNT == 0
+  virtual void invalidate();		// when transaction ends
+  virtual int ready();			// can delete bridge now?
+};
+
 struct osp_txn;
 
 // per-thread globals
@@ -17,12 +26,11 @@ struct osp_thr {
   int tie_objects;
   struct osp_txn *txn;
   tix_handler handler;
-  ossv_bridge *bridge_top;   //should be invalid
+  osp_bridge *bridge_top;   //should be invalid
 
   //methods
-  int can_update();
+  int can_update(void *);
   void destroy_bridge();
-//  void invalidate(OSSVPV *pv);
 
   //glue methods
   os_segment *sv_2segment(SV *);
@@ -42,16 +50,16 @@ struct osp_txn {
   ~osp_txn();
   void abort();
   void commit();
-//  void invalidate(OSSVPV *pv);
   void post_transaction();
-  int can_update();
+  int can_update(os_database *);
+  int can_update(void *);
   void prepare_to_commit();
   int is_prepare_to_commit_invoked();
   int is_prepare_to_commit_completed();
 
   os_transaction::transaction_type_enum tt;
   osp_txn *up;
-  ossv_bridge *bridge_top;
+  osp_bridge *bridge_top;
   os_transaction *os;
   tix_handler handler;
   int got_os_exception;
